#!/usr/bin/env python
#coding:utf-8
def get_output(cmd):
    '''
    好像是有的python版本的subprocess不存在check_output,所以要自己判断和封装一下
    利用commands.getoutput
    '''
    try:
        from subprocess import check_output
        return check_output(cmd,shell=True)
    except ImportError:
        from commands import getoutput
        return getoutput(cmd)


#!/usr/bin/env python
#coding=utf-8
import os
import sys
import ConfigParser
import subprocess
from ngsubprocess import get_output

'''
check python version,if version lt 2.5,mini-ng will not be run
'''
py   = sys.version_info
py25 = py <  (2, 5, 0)
if py25:
    print 'python version is need great than 2.5'
    sys.exit(3)
PY_PATH=''
if subprocess.call('which python2.7 >/dev/null 2>&1 || which python27 >/dev/null 2>&1',shell=True) == 0:
    PY_PATH = get_output('which python2.7 2>/dev/null || which python27')
elif subprocess.call('which python2.6 >/dev/null 2>&1 || which python26 >/dev/null 2>&1',shell=True) == 0:
    PY_PATH = get_output('which python2.6 2>/dev/null || which python26 2>/dev/null')
PY_PATH = PY_PATH.replace('\n','')
'''
    定义程序的名称，版本，更新日期，作者信息
'''
APP_NAME = 'ng-mini'
APP_VERSION = 0.1
APP_AUTHOR = 'qing.chen'

'''
    处理程序的路径,可能需要手动更改
'''
APP_PATH = '/data/ng-mini'
#APP_PATH = os.path.abspath('.')

CONFIG_PATH = os.path.join(APP_PATH ,'etc')
'''
    配置文件解析
'''
cf = ConfigParser.ConfigParser()
cf.read(CONFIG_PATH + '/ng.conf')
#是不是要记录日志
LOGED = cf.get('log','log')
#日志的记录路径是什么，默认在程序目录下的log目录
try:
    LOG_PATH = cf.get('log','logpath')
except:
    LOG_PATH = APP_PATH + '/log/ng.log'

#日志级别
try:
    LOG_LEVEL = cf.get('log','log_level').upper()
except:
    LOG_LEVEL = 'INFO'
#rrdtool 文件放在了哪里
try:
    RRDTOOL_PATH = cf.get('base','rrdtool_path')
except:
    RRDTOOL_PATH = get_output('/usr/bin/which rrdtool')

if not os.path.isfile(RRDTOOL_PATH):
    RRDTOOL_PATH = get_output('/usr/bin/which rrdtool')

RRDTOOL_PATH = RRDTOOL_PATH.replace('\n','')


#pid文件
try:
    PID = cf.get('base','pid')
except:
    PID = '/var/run/ng.pid'
#yaml文件的路径
try:
    YAML_PATH = CONFIG_PATH + '/' + cf.get('base','yaml_path')
except:
    YAML_PATH = ' '

#需要作图的app
try:
    APPS = cf.get('apps','enabled').split(' ')
except:
    APPS = ['base']

#rrd 文件路径

try:
    RRD_PATH = os.path.join(APP_PATH,cf.get('base','rrd_path'))
except:
    RRD_PATH = APP_PATH + '/' + 'rrd'

#font 字体文件路径
try:
    FONT_PATH = os.path.join(APP_PATH,'font')
except:
    FONT_PATH = APP_PATH + '/' + 'font'

#图片文件存放路径

try:
    PIC_PATH = os.path.join(APP_PATH,cf.get('base','graph_path'))
except:
    PIC_PATH = APP_PATH + '/' + 'web/pic/rrd'

#apps 自定义数据获取脚本的路径
try:
    APPS_PATH = os.path.join(APP_PATH,cf.get('base','apps_path'))
except:
    APPS_PATH = APP_PATH + '/' + 'apps'

#mysql database config
try:
    MYSQL_HOST=cf.get('mysql','host')
    MYSQL_USER=cf.get('mysql','user')
    MYSQL_PASSWD=cf.get('mysql','passwd')
    MYSQL_DATABASE=cf.get('mysql','database')
except:
    MYSQL_HOST=''
    MYSQL_USER=''
#mongodb database config
try:
    MONGO_HOST=cf.get('mongodb','host')
    MONGO_USER=cf.get('mongodb','user')
    MONGO_PORT=int(cf.get('mongodb','port'))
    MONGO_PASSWD=cf.get('mongodb','passwd')
    MONGO_DATABASE=cf.get('mongodb','database')
except:
    MONGO_HOST=''
    MONGO_USER=''



#!/usr/bin/env python
#coding:utf-8
import yaml
import nginit
from  nglog import log

def ngyaml(filename):
    '''
    解析yaml配置文件
    '''
    d = {}
    try:
        f = open(filename)
        d = yaml.safe_load(f)
    except:
        log.error('can\'t open the yaml file %s' %(filename))
    return d



#!/usr/bin/env python
#coding:utf-8
import logging
import logging.handlers
import nginit

def nglog():
    '''
    处理ng的日志相关的函数
    '''
    if nginit.LOGED:
        logger = logging.getLogger()
        logger.setLevel(getattr(logging,nginit.LOG_LEVEL))
        File = logging.handlers.RotatingFileHandler(nginit.LOG_PATH,maxBytes=1024*1024,backupCount=5)
        Format = logging.Formatter("%(asctime)s   %(levelname)s - %(message)s","%Y-%m-%d %H:%M:%S")
        File.setFormatter(Format)
        logger.addHandler(File)
        return logger

log = nglog()





#!/usr/bin/env python
#coding:utf-8
#import pyrrd //考虑到通用性还是使用封装命令行的方法吧
import subprocess
import os
import time
import nginit
from nglog import log
from ngyaml import ngyaml
from ngsubprocess import get_output

rrdtool = nginit.RRDTOOL_PATH

def create(filename,rrd_ds,rrd_dst,interval):
    '''
    rrdtool create in python
    '''
    start = int(time.time()) - 86400
    step = interval * 60
    cmd = "%s  create %s \
    --start %s --step %s \
    DS:%s:%s:600:0:U  \
    RRA:LAST:0.5:1:600 RRA:LAST:0.5:6:700 RRA:LAST:0.5:24:775 RRA:LAST:0.5:288:797 \
    RRA:AVERAGE:0.5:1:600 RRA:AVERAGE:0.5:6:700 RRA:AVERAGE:0.5:24:775 RRA:AVERAGE:0.5:288:797\
    RRA:MAX:0.5:1:600 RRA:MAX:0.5:6:700 RRA:MAX:0.5:24:775 RRA:MAX:0.5:444:797 \
    RRA:MIN:0.5:1:600 RRA:MIN:0.5:6:700 RRA:MIN:0.5:24:775 RRA:MIN:0.5:444:797  " %(rrdtool,filename,start,step,rrd_ds,rrd_dst)
    try:
        subprocess.call(cmd, shell=True)
    except:
        log.error('create rrd file %s failed' %(filename))


def update(filename,value):
    '''
    rrdtool update function
    '''
    cmd = "%s update %s \
    N:%s" %(rrdtool,filename,value)
    try:
        subprocess.call(cmd, shell=True)
    except:
        log.error('update rrd file %s failed' %(filename))


def graph(filename,rrd_file,rrd_ds,rrd_color):
    '''
    rrdtool graph function
    '''
    cmd = "%s graph %s \
                    DEF:v1=%s:%s:AVERAGE \
                    AREA:v1#%s" %(rrdtool,filename,rrd_file,rrd_ds,rrd_color)
    try:
        subprocess.call(cmd, shell=True)
    except:
        log.error('graph rrd file %s failed' %(filename))

def graph_combain(app_name,title,vertical,date='day'):
    '''
    the app_name is also the yaml file's name,title is the rrdtool pic's title
    '''
    dt = {'day':86400,'week':604800,'month':2592000,'year':31536000}
    if date not in ['day','week','month','year']:
        date = 'day'
    font = nginit.FONT_PATH + '/' + 'msyh.ttf'
    graph_header = '%s graph %s/ng-mini-%s.png -w 600 -h 300 \
    -t "%s" -v "%s" \
    -n TITLE:12:%s  -n LEGEND:8:%s \
    --start -%s --end now \
    --slope-mode \
    --upper-limit 1500 \
    -Y  --base=1000   --upper-limit 1500      -l 0 ' %(nginit.RRDTOOL_PATH,nginit.PIC_PATH,date,title,vertical,font,font,dt[date])
    if date == 'day':
        graph_header += '-x MINUTE:30:HOUR:1:HOUR:1:0:"%H" '

    yaml_file = nginit.YAML_PATH +'/' + app_name + '.yaml'
    app = ngyaml(yaml_file)
    graph_body_top = ''
    graph_body_center = ''
    graph_body_bottom = ''
    DS = []
    DEF = []
    CDEF = []
    METHOD = []
    COLOR = []
    RRA = ['LAST','MAX','AVERAGE','MIN']
    i = 0
    for a in app:
        if a['CreatePic']:
            rrd_file = nginit.RRD_PATH + '/' + app_name + '_' +a['Info'] + '.rrd'
            rrd_ds = a['RrdDs']
            DS.append(rrd_ds)
            METHOD.append(a['RrdMethod'].upper())
            COLOR.append('#'+a['Color'])
            DEF.append('DEF:v'+str(i)+'='+rrd_file+':'+rrd_ds+':AVERAGE')
            rrd_range = str(a['Range']).split(',')[0]
            if rrd_range.startswith('-'):
                if a['Info'].startswith('BAND_WIDTH'):
                    CDEF.append('CDEF:t'+str(i)+'=v'+str(i)+',8,*, ' + str(rrd_range).replace('-','')+', - ')
		else:
                    CDEF.append('CDEF:t'+str(i)+'=v'+str(i)+','+str(rrd_range).replace('-','')+', - ')
            else:
                if a['Info'].startswith('BAND_WIDTH'):
                    CDEF.append('CDEF:t'+str(i)+'=v'+str(i)+',8,*,' + str(rrd_range)+',+ ')
		else:
                    CDEF.append('CDEF:t'+str(i)+'=v'+str(i)+','+str(rrd_range)+',+  ')
            graph_body_top += str(DEF[i] +' '+CDEF[i] )
            if a['Info'].startswith('BAND_WIDTH'):
            	graph_body_bottom += str(METHOD[i]+':t'+str(i)+COLOR[i]+':'+ '\" ' + DS[i] + '\"' +\
                                   ' GPRINT:t'+str(i)+':LAST:"LAST %6.2lf %Sbps" '+\
                                    ' GPRINT:t'+str(i)+':MAX:"MAX %6.2lf %Sbps" '+\
                                   ' GPRINT:t'+str(i)+':AVERAGE:"AVERAGE %6.2lf %Sbps" '+\
                                   ' GPRINT:t'+str(i)+':MIN:"MIN %6.2lf %Sbps" '+' COMMENT:"\\n" ')
            else:
            	graph_body_bottom += str(METHOD[i]+':t'+str(i)+COLOR[i]+':'+ '\" ' + DS[i] + '\"' +\
                                   ' GPRINT:v'+str(i)+':LAST:"LAST %6.2lf " '+\
                                    ' GPRINT:v'+str(i)+':MAX:"MAX %6.2lf " '+\
                                   ' GPRINT:v'+str(i)+':AVERAGE:"AVERAGE %6.2lf " '+\
                                   ' GPRINT:v'+str(i)+':MIN:"MIN %6.2lf " '+' COMMENT:"\\n" ')
            	
            i = i+1
            if i >len(app):
                break
    graph_total = graph_header + graph_body_top + ' ' + graph_body_center + ' ' + graph_body_bottom
    subprocess.call(str(graph_total),shell=True)

def graph_combain_without_network(app_name,title,vertical,date='day'):
    '''
    the app_name is also the yaml file's name,title is the rrdtool pic's title
    '''
    dt = {'day':86400,'week':604800,'month':2592000,'year':31536000}
    if date not in ['day','week','month','year']:
        date = 'day'
    font = nginit.FONT_PATH + '/' + 'msyh.ttf'
    graph_header = '%s graph %s/ng-mini-%s.png -w 600 -h 300 \
    -t "%s" -v "%s" \
    -n TITLE:12:%s  -n LEGEND:8:%s \
    --start -%s --end now \
    --slope-mode \
    -Y -X 0        -l 0 ' %(nginit.RRDTOOL_PATH,nginit.PIC_PATH,date,title,vertical,font,font,dt[date])
    if date == 'day':
        graph_header += '-x MINUTE:30:HOUR:1:HOUR:1:0:"%H" '

    yaml_file = nginit.YAML_PATH +'/' + app_name + '.yaml'
    app = ngyaml(yaml_file)
    graph_body_top = ''
    graph_body_center = ''
    graph_body_bottom = ''
    DS = []
    DEF = []
    CDEF = []
    METHOD = []
    COLOR = []
    RRA = ['LAST','MAX','AVERAGE','MIN']
    i = 0
    for a in app:
        if a['CreatePic'] and not a['Info'].startswith('BAND_WIDTH'):
            rrd_file = nginit.RRD_PATH + '/' + app_name + '_' +a['Info'] + '.rrd'
            rrd_ds = a['RrdDs']
            DS.append(rrd_ds)
            METHOD.append(a['RrdMethod'].upper())
            COLOR.append('#' + str(a['Color']))
            DEF.append('DEF:v'+str(i)+'='+rrd_file+':'+rrd_ds+':AVERAGE')
            rrd_range = str(a['Range']).split(',')[0]
            if rrd_range.startswith('-'):
                CDEF.append('CDEF:t'+str(i)+'=v'+str(i)+','+str(rrd_range).replace('-','')+', - ')
            else:
                CDEF.append('CDEF:t'+str(i)+'=v'+str(i)+','+str(rrd_range)+',+  ')
            graph_body_top += str(DEF[i] +' '+CDEF[i] )
            graph_body_bottom += str(METHOD[i]+':t'+str(i)+COLOR[i]+':'+ '\" ' + DS[i] + '\"' +\
                                   ' GPRINT:v'+str(i)+':LAST:"LAST %6.2lf " '+\
                                    ' GPRINT:v'+str(i)+':MAX:"MAX %6.2lf " '+\
                                   ' GPRINT:v'+str(i)+':AVERAGE:"AVERAGE %6.2lf " '+\
                                   ' GPRINT:v'+str(i)+':MIN:"MIN %6.2lf " '+' COMMENT:"\\n" ')

            i = i+1
            if i >len(app):
                break
    graph_total = graph_header + graph_body_top + ' ' + graph_body_center + ' ' + graph_body_bottom
    subprocess.call(str(graph_total),shell=True)

def graph_network(app_name,title,vertical,date='day'):
    '''
    the app_name is also the yaml file's name,title is the rrdtool pic's title
    '''
    dt = {'day':86400,'week':604800,'month':2592000,'year':31536000}
    if date not in ['day','week','month','year']:
        date = 'day'
    font = nginit.FONT_PATH + '/' + 'msyh.ttf'
    graph_header = '%s graph %s/ng-mini-network-%s.png -w 600 -h 300 \
    -t "%s" -v "%s" \
    -n TITLE:12:%s  -n LEGEND:8:%s \
    --start -%s --end now \
    --slope-mode \
    --upper-limit 1500 \
    -Y  --base=1000   --upper-limit 1500      -l 0 ' %(nginit.RRDTOOL_PATH,nginit.PIC_PATH,date,title,vertical,font,font,dt[date])
    if date == 'day':
        graph_header += '-x MINUTE:30:HOUR:1:HOUR:1:0:"%H" '

    yaml_file = nginit.YAML_PATH +'/' + app_name + '.yaml'
    app = ngyaml(yaml_file)
    graph_body_top = ''
    graph_body_center = ''
    graph_body_bottom = ''
    DS = []
    DEF = []
    CDEF = []
    METHOD = []
    COLOR = []
    RRA = ['LAST','MAX','AVERAGE','MIN']
    i = 0
    for a in app:
        if a['CreatePic'] and a['Info'].startswith('BAND_WIDTH'):
            rrd_file = nginit.RRD_PATH + '/' + app_name + '_' +a['Info'] + '.rrd'
            rrd_ds = a['RrdDs']
            DS.append(rrd_ds)
            METHOD.append(a['RrdMethod'].upper())
            COLOR.append('#'+a['Color'])
            DEF.append('DEF:v'+str(i)+'='+rrd_file+':'+rrd_ds+':AVERAGE')
            rrd_range = str(a['Range']).split(',')[0]
            if rrd_range.startswith('-'):
                CDEF.append('CDEF:t'+str(i)+'=v'+str(i)+',8,*, ' + str(rrd_range).replace('-','')+', - ')
            else:
                CDEF.append('CDEF:t'+str(i)+'=v'+str(i)+',8,*,' + str(rrd_range)+',+ ')
            graph_body_top += str(DEF[i] +' '+CDEF[i] )
            graph_body_bottom += str(METHOD[i]+':t'+str(i)+COLOR[i]+':'+ '\" ' + DS[i] + '\"' +\
                                   ' GPRINT:t'+str(i)+':LAST:"LAST %6.2lf %Sbps" '+\
                                    ' GPRINT:t'+str(i)+':MAX:"MAX %6.2lf %Sbps" '+\
                                   ' GPRINT:t'+str(i)+':AVERAGE:"AVERAGE %6.2lf %Sbps" '+\
                                   ' GPRINT:t'+str(i)+':MIN:"MIN %6.2lf %Sbps" '+' COMMENT:"\\n" ')
            	
            i = i+1
            if i >len(app):
                break
    graph_total = graph_header + graph_body_top + ' ' + graph_body_center + ' ' + graph_body_bottom
    subprocess.call(str(graph_total),shell=True)

if __name__ == '__main__':
    yaml_file = nginit.YAML_PATH +'/base.yaml'
    app = ngyaml(yaml_file)
    graph_combain('base',time.strftime("%Y-%m-%d-%H:%M:%S",time.localtime()),'汇总图','day')






#!/usr/bin/python
from setuptools import setup , find_packages

setup(
        name = 'ng-mini',
        version = '0.1.0',
        long_description=open("README.md").read(),
        packages = find_packages(),
        requires = ['bottle','PyYAML'],
        author = 'qing.chen',
        url = 'http://www.chenqing.org'
        )



#!/usr/bin/env python
import sys
import os
import subprocess

import nginit
from nglog import log

def report_mysql(data):
    """
    report collect data to mysql .
    need MySQL-python module
    note that:
        host is your mysql server ip or hostname
        user is your mysql user
        passwd is your mysql password
        database is the database store collect data
        data is a dict (note:is dict),the key is mysql table's key ,and the
            value of the dict is your collect data or other thing 
    """
    try:
        import MySQLdb
    except ImportError:
        log.error('MySQLdb module is not install in this server !')
        sys.exit(200)
    sql = 'insert into `' + table + '` ('
    keys = []
    values = []
    
    try:
        conn = MySQLdb.connect(nginit.MYSQL_HOST,nginit.MYSQL_USER,nginit.MYSQL_PASSWD)
        conn.select_db(nginit.MYSQL_DATABASE)
        cursor = conn.cursor()
    except:
        log.error('connect mysql server or database  error !')
    if isinstance(data,dict):
        for (key,value) in data.items():
           keys.append(key)
           values.append(value)
        for i in range(len(keys)):
            if i < len(keys) - 1 :
                sql += '`' + str(keys[i]) + '`'  + ','
            else:
                sql += '`' + str(keys[i]) + '`  ) values ('
        for j in range(len(values)):
            if j < len(values) - 1:
                 sql += '"' + str(values[j]) + '"'  + ','
            else:
                 sql += '"' + str(values[j]) + '" )' 
    else:
        log.error(' input data not a dict')

    try:        
        cursor.execute(sql)
        conn.commit()
        log.info('report ' + str(data) + ' to ' + host + ' success')
    except:
        log.error('insert sql '+ sql +' error')
def report_mongo(data):
    """
    report collect data to mongodb
    need pymongo library
    """
    try:
        import pymongo
    except:
        log.error('pymongo library is not installed in this server')
    import pymongo
    #connect your mongo server 
    conn = pymongo.MongoClient('mongodb://user:password@localhost:27017')
    #get or create a new database
    db = conn.ng_mini
    #check auth
    #db.authenticate(nginit.MONGO_USER,nginit.MONGO_PASSWD)
    #select a collecton (a table),assume your collection name is monitor
    collection = db.monitor
    if isinstance(data,dict):
        try:
            collection.insert(data)
            log.info('report ' + str(data) + 'to' + nginit.MONGO_HOST + ' success')
        except:
            log.error('report ' + str(data) + 'to' + nginit.MONGO_HOST + ' failed')
    else:
        log.error('data not a dict !')


def report_redis():
    """
    report collect data to redis
    """


def report_memcache():
    """
    report collect data to memcache
    """

def report_http_post():
    """
    report collect data use http post
    """

if __name__ == '__main__':
    data = {'hostname':'chenqing.org','load':18}
    #report_mysql(host,user,passwd,database,'test',data)
    report_mongo(data)
